'''
 модуль финальной оценкb данных, получаемых после работы модуля analise
 основные используемые классы: абстрактный класс Evaluator и его наследники (RandomForestEval, MLPEval, ...).
 основная функция - evaluate, 
 на вход получает данные (в формате DataFrame) со значениями рассчитанных количественных характеристик,
 где каждая строка соответствует отдельной записи или отрезку записи оператора.
 на выход - результат оценки (классификации) каждой строки 
'''
