def eng_to_rus(name):
    if name == 'File':  return 'Файл'

    if name == 'x_mean':  return 'Усредненная позиция взгляда по оси X'
    if name == 'x_std':  return 'Среднее квадратичное отклонение позиции взгляда по оси X'
    if name == 'x_min':  return 'Минимальная позиция взгляда по оси X'
    if name == 'x_max':  return 'Максимальная позиция взгляда по оси X'
    if name == 'x_25':  return 'Процентиль 25 позиции взгляда по оси X'
    if name == 'x_50':  return 'Процентиль 50 позиции взгляда по оси X'
    if name == 'x_75':  return 'Процентиль 75 позиции взгляда по оси X'
    if name == 'y_mean':  return 'Усредненная позиция взгляда по оси Y'
    if name == 'y_std':  return 'Среднее квадратичное отклонение позиции взгляда по оси Y'
    if name == 'y_min':  return 'Минимальная позиция взгляда по оси Y'
    if name == 'y_max':  return 'Максимальная позиция взгляда по оси Y'
    if name == 'y_25':  return 'Процентиль 25 позиции взгляда по оси Y'
    if name == 'y_50':  return 'Процентиль 50 позиции взгляда по оси Y'
    if name == 'y_75':  return 'Процентиль 75 позиции взгляда по оси Y'
                            
    if name == 'False Fixation, per minute':  return 'Обособленные фиксации, количество в минуту'
    if name == 'False Saccades, per minute':  return 'Сверхкороткие саккады, количество в минуту'
    if name == 'Saccades with amplitude > 6 degrees, per minute':  return 'Саккады с амплитудой больше 6 градусов, количество в минуту'
    if name == 'Saccades with amplitude < 6 degrees, per minute':  return 'Саккады с амплитудой меньше 6 градусов, количество в минуту'
    if name == 'Max Curve':  return 'Максимальная кривизна траектории саккады'
    if name == 'Average Curve':  return 'Средняя кривизна траектории саккады'
    if name == 'Min Curve':  return 'Минимальная кривизна траектории саккады'
    if name == 'Time Frame':  return 'Время записи'
    if name == 'Saccades time':  return 'Суммарная длительность саккад'
    if name == 'Fixation time < 150 ms':  return 'Суммарная длительность фиксаций короче 150мс'
    if name == 'Fixation time > 150 ms':  return 'Суммарная длительность фиксаций длиннее 150мс'
    if name == 'Fixation time between 150 and 900 ms':  return 'Суммарная длительность фиксаций длиннее 150мс но короче 900мс'
    if name == 'Fixation time > 900 ms':  return 'Суммарная длительность фиксаций длиннее 900мс'
    if name == 'Fixation time < 180 ms':  return 'Суммарная длительность фиксаций короче 180мс'
    if name == 'Fixation time > 180 ms':  return 'Суммарная длительность фиксаций длиннее 180мс'
    if name == '% of Fixations < 150 ms':  return 'Доля фиксаций короче 150мс от всех фиксаций, в %'
    if name == '% of Fixations > 150 ms':  return 'Доля фиксаций длиннее 150мс от всех фиксаций, в %'
    if name == '% of Fixations between 150 and 900 ms':  return 'Доля фиксаций длиннее 150мс но короче 900мс от всех фиксаций, в %'
    if name == '% of Fixations > 900 ms':  return 'Доля фиксаций длиннее 900мс от всех фиксаций, в %'
    if name == '% of Fixations < 180 ms':  return 'Доля фиксаций короче 180мс от всех фиксаций, в %'
    if name == '% of Fixations > 180 ms':  return 'Доля фиксаций длиннее 180мс от всех фиксаций, в %'
    if name == 'Fixation time < 150 ms, per time':  return 'Доля времени (от общего времени записи) фиксаций короче 150мс'
    if name == 'Fixation time > 150 ms, per time':  return 'Доля времени (от общего времени записи) 150мс'
    if name == 'Fixation time between 150 and 900 ms, per time':  return 'Доля времени (от общего времени записи) фиксаций длиннее 150мс но короче 900мс'
    if name == 'Fixation time > 900 ms, per time':  return 'Доля времени (от общего времени записи) фиксаций длиннее 900мс'
    if name == 'Fixation time < 180 ms, per time':  return 'Доля времени (от общего времени записи) фиксаций короче 180мс'
    if name == 'Fixation time > 180 ms, per time':  return 'Доля времени (от общего времени записи) фиксаций длиннее 180мс'
    if name == "% of Fixations < 150 ms, per minute":  return 'Количество фиксаций короче 150мс на минуту записи'
    if name == '% of Fixations > 150 ms, per minute':  return 'Количество фиксаций длиннее 150мс на минуту записи'
    if name == '% of Fixations between 150 and 900 ms, per minute':  return 'Количество фиксаций длиннее 150мс но короче 900мс на минуту записи'
    if name == '% of Fixations > 900 ms, per minute':  return 'Количество фиксаций длиннее 900мс на минуту записи'
    if name == '% of Fixations < 180 ms, per minute':  return 'Количество фиксаций короче 180мс на минуту записи'
    if name == '% of Fixations > 180 ms, per minute':  return 'Количество фиксаций длиннее 180мс на минуту записи'
    if name == 'Fixation Frequency':  return 'Частота возникновения фиксаций'
    if name == 'Average Fix Frequency in interval (1s)':  return 'Средняя частота возникновения фиксаций в секундном интервале'
    if name == 'Max Fix Frequency in interval (1s)':  return 'Максимальная частота возникновения фиксаций в секундном интервале'
    if name == 'Average Acceleration':  return 'Средний модуль ускорения движения взгляда'
    if name == 'Min Acceleration':  return 'Минимальный модуль ускорения движения взгляда'
    if name == 'Max Acceleration':  return 'Максимальная модуль ускорения движения взгляда'
    if name == 'Average Speed':  return 'Средняя скорость движения взгляда'
    if name == 'Min Speed':  return 'Минимальная скорость движения взгляда'
    if name == 'Max Speed':  return 'Максимальная скорость движения взгляда'
    if name == 'Average Acceleration in interval (1s)':  return 'Средний модуль ускорения движения взгляда в секундном интервале'
    if name == 'Min Acceleration in interval (1s)':  return 'Минимальный модуль ускорения движения взгляда в секундном интервале'
    if name == 'Max Acceleration in interval (1s)':  return 'Максимальный модуль ускорения движения взгляда в секундном интервале'
    if name == 'Average Speed in interval (1s)':  return 'Средняя скорость движения взгляда в секундном интервале'
    if name == 'Min Speed in interval (1s)':  return 'Минимальная скорость движения взгляда в секундном интервале'
    if name == 'Max Speed in interval (1s)':  return 'Максимальная скорость движения взгляда в секундном интервале'
    if name == 'Average Fixation Speed':  return 'Средняя скорость движения взгляда в фиксациях'
    if name == 'Min Fixation Speed':  return 'Минимальная скорость движения взгляда в фиксациях'
    if name == 'Max Fixation Speed':  return 'Максимальная скорость движения взгляда в фиксациях'
    
    if name == 'Average Fixation Speed, < 150ms':  return 'Средняя скорость движения взгляда в фиксациях короче 150мс'
    if name == 'Min Fixation Speed, < 150ms':  return 'Минимальная скорость движения взгляда в фиксациях короче 150мс'
    if name == 'Max Fixation Speed, < 150ms':  return 'Максимальная скорость движения взгляда в фиксациях короче 150мс'                      
    if name == 'Average Fixation Speed, > 150ms':  return 'Средняя скорость движения взгляда в фиксациях длинее 150мс'
    if name == 'Min Fixation Speed, > 150ms':  return 'Минимальная скорость движения взгляда в фиксациях длинее 150мс'
    if name == 'Max Fixation Speed, > 150ms':  return 'Максимальная скорость движения взгляда в фиксациях длинее 150мс'
    if name == 'Average Saccade Speed':  return 'Средняя скорость движения взгляда в саккадах'
    if name == 'Min Saccade Speed':  return 'Минимальная скорость движения взгляда в саккадах'
    if name == 'Max Saccade Speed':  return 'Максимальная скорость движения взгляда в саккадах'
    if name == 'Average Saccade Length':  return 'Средняя длина саккады'
    if name == 'Min Saccade Length':  return 'Минимальная длина саккады'
    if name == 'Max Saccade Length':  return 'Максимальная длина саккады'
    if name == 'Average Saccade Time':  return 'Средняя длительность саккады'
    if name == 'Min Saccade Time':  return 'Минимальная длительность саккады'
    if name == 'Max Saccade Time':  return 'Максимальная длительность саккады'


                          
    print(name, 'have no rus name!')
    return name